/*************************************************************************************
Data Visualization - Final Project - Health in U.S. Counties
Tested in Processing 3                                    
                                                     
Author: Sarah Wells                      
Supervisor: George Legrady                           

Purpose:  Visualize the relationship between a variety of health factors and 
the life expectancies in America's counties.

Usage:    1. A mouse left-drag will rotate the camera around the subject.
          2. A right drag will zoom in and out. 
          3. A middle-drag (command-left-drag on mac) will pan. 
          4. A double-click restores the camera to its original position. 
          5. The shift key constrains rotation and panning to one axis or the other.
          6. Select factors on the left - changing factors will stop the organization.
          7. Press Reset/Run/Step to reorganize data with new selected factors.
          8. Mouse Over a point to view details.
*************************************************************************************/
import peasy.*;
import controlP5.*;
import java.util.*;

PeasyCam cam;
ControlP5 cp5;
KohonenSet klist = new KohonenSet();

//global variables
PImage colorBar, scaleBar;
float tempList[][];
Table table;
String[] factorNames;
//float [] factors;
int[] factorCols;
int numRows, numCols, numSelectedFactors, nodes, nodes3D;
//precalculated
int numFactors = 36;
int numCounties = 1742;
float[] maxValues, minValues;
int[] selectedFactors, selectedCounties;
int tempInt;
boolean tempBoolean;
String state, county;
ArrayList <County> counties = new ArrayList<County>();
//boolean temp;
boolean[] useFactors, mouseFactor;
boolean run;
void setup(){
  size(1000, 800, P3D);
  //set camera
  cam = new PeasyCam(this, 3000);
  cp5 = new ControlP5(this);
  cam.setWheelScale(0.05);
  colorMode(HSB);
  textMode(MODEL);
  //load data
  table = loadTable("All_Health_Data.csv");
  colorBar = loadImage("red_green_gradient.png");
  scaleBar = loadImage("green_red_gradient.png");
  //2 bottom/footer lines
  numRows = table.getRowCount()-2;
  numCols = table.getColumnCount();
  factorCols = new int[numFactors];
  //read in which col factors are in
  tempInt = 0;
  for(int i=0; i<numCols; i++){
    if(table.getString(0,i).length()>0){
    factorCols[tempInt]=i;
    tempInt++;
    }
  }
  maxValues= new float[numFactors];
  minValues= new float[numFactors];
  useFactors = new boolean[numFactors];
  mouseFactor = new boolean[numFactors];
  //factors = new float[numFactors];
  //loop through to store factor names, max, min
  factorNames = new String[numFactors];
  //tempInt = 0;
  for(int i=0; i<numFactors; i++){
    factorNames[i]=table.getString(1,factorCols[i]);
    minValues[i] = 1000000000; //arbitrarily big
    maxValues[i] = 0; //small
    useFactors[i] = false; //default to display none
    mouseFactor[i] = false; //default not on factors
  }
  
  //loop through to create counties and get county factors
  //calculate max and mins
  for( int i=2; i< numRows; i++){
    tempBoolean = true;
    float[] factors = new float[numFactors];
    for(int j=0; j< numFactors; j++){
      //check every factor exists
      if(!(table.getString(i,factorCols[j]).length()>0)){
        tempBoolean=false;
      }
      else{
        state = table.getString(i,1);
        county = table.getString(i,2);
        factors[j] = table.getFloat(i,factorCols[j]);
        minValues[j] = min(minValues[j], factors[j]);
        maxValues[j] = max(maxValues[j], factors[j]);
      }
    }
    if(tempBoolean){
      counties.add(new County( state, county, factors));
    }
  }
  createList();
  klist.kset = tempList;
  nodes = (int)(2*sqrt(numCounties));
  
  //try this 
  //nodes = (int) (4*sqrt(numCounties));
  
  //println(numCounties);
  //1742^(1/3) rounded up = 13
  //4^(1/3) = 1.5874
  //nodes3D = (int) (1.5874)*13;
  klist.initWeights(nodes,numFactors);

  nodes3D = 13;
  //klist.initWeights3D(nodes3D,numFactors);
  
  //select some arbitraty Factors
  useFactors[1] = true;
  useFactors[2] = true;
  useFactors[3] = true;
  useFactors[30] = true;
  //set to run
  run = true;
}

void draw(){
  background(0);
    //do things
    legend();
    if(run){
    //klist.initWeights(nodes,numFactors);
    klist.train(useFactors);
    //klist.initWeights3D(nodes,numFactors);
    //klist.train3D(useFactors);
    }
    klist.drawText(useFactors, counties, factorNames);
    //klist.drawText3D(useFactors, counties, factorNames);

    //klist.drawPoints(useFactors, counties);
}

void createList(){
  tempList = new float[numCounties][numFactors]; // each color has three attributes 
  for (int i=0; i<numCounties; i++){
    for (int j=0; j<numFactors; j++){
      tempList[i][j] = map(counties.get(i).factors[j], minValues[j], maxValues[j], 0, 100);
      if(j==0){
        //println(counties.get(i).factors[j]);
      //println( "i " + i + " j " + j + " tempList " + tempList[i][j]);
      }
    }
  }
}



void legend() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  cp5.draw();
  
  //textMode(SHAPE);
  
  //side labels
  for(int i = 0; i<(numFactors-1); i++){
    stroke(255);
    strokeWeight(2);
    if(!(useFactors[i])){
    noFill();
    }
    else{
    fill(255);
    }
    ellipse(width-217,100+20*i,15,15);
     // Test if the cursor is over the box 
    if(mouseX > width-217 && mouseX < width-132 &&  mouseY > 95+20*(i) && mouseY < 95+20*(i+1)) {
      mouseFactor[i] = true;  
      fill(204, 89, 98);
    } 
    else{
      fill(255);
      mouseFactor[i] = false;
    }
    //fill(255);
    textAlign(LEFT);
    textSize(12);
    text(factorNames[i],width-197,103+i*20);
  }
 
    //stop run titles - list as last mouseFactor
    stroke(255);
    strokeWeight(2);
    if(run){
    noFill();
    }
    else{
    fill(255);
    }
    ellipse(width-150,50,25,25);
     // Test if the cursor is over the box 
    if(mouseX > width-150 && mouseX < width-80 &&  mouseY > 25 && mouseY < 75){
      mouseFactor[numFactors-1] = true;  
      fill(204, 89, 98);
    } 
    else{
      mouseFactor[numFactors-1] = false;
      fill(255);
    }
    //fill(255);
    textAlign(LEFT);
    textSize(15);
    text("Reset&Run/Stop",width-130,55);

    
  //title
  fill(255);
  textAlign(LEFT);
  textSize(18);
  text("Life Expectancy & Health Factors Relationship By U.S. County",width*2/9,30);
  textSize(10);
  text("Sarah Wells, MAT 259", width*2/9, 50);
  
  //life expectancy scale
  image(scaleBar, 20, 20, 150, 12);
  textSize(12);
  text("Life Expectancy Scale (years)", 20,15);
  text(minValues[numFactors-1], 10, 44);
  text(maxValues[numFactors-1], 150, 44);
  
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);


}

void mousePressed(){
  for(int i=0; i<(numFactors-1); i++){
    if(mouseFactor[i]){
      useFactors[i] = !useFactors[i];
      //stops run
      run = false;
    }
  }
  if(mouseFactor[numFactors-1]){
      run = !run;
      if(run){
          klist.initWeights(nodes,numFactors);
          klist.reset();
      }
  }
}
